package javax.lang.model.util;

import javax.lang.model.element.TypeParameterElement;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.VariableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.PackageElement;
import java.util.Iterator;
import javax.lang.model.element.Element;
import javax.lang.model.SourceVersion;
import javax.annotation.processing.SupportedSourceVersion;

@SupportedSourceVersion(SourceVersion.RELEASE_6)
public class ElementScanner6<R, P> extends AbstractElementVisitor6<R, P>
{
    protected final R DEFAULT_VALUE;
    
    protected ElementScanner6() {
        this.DEFAULT_VALUE = null;
    }
    
    protected ElementScanner6(R default_VALUE) {
        this.DEFAULT_VALUE = default_VALUE;
    }
    
    public final R scan(Iterable<? extends Element> iterable, P p2) {
        Object o = this.DEFAULT_VALUE;
        Iterator<Object> iterator = iterable.iterator();
        while (iterator.hasNext()) {
            o = this.scan((Element)iterator.next(), p2);
        }
        return o;
    }
    
    public R scan(Element element, P p2) {
        return element.accept(this, p2);
    }
    
    public final R scan(Element element) {
        return this.scan(element, null);
    }
    
    @Override
    public R visitPackage(PackageElement packageElement, P p2) {
        return this.scan(packageElement.getEnclosedElements(), p2);
    }
    
    @Override
    public R visitType(TypeElement typeElement, P p2) {
        return this.scan(typeElement.getEnclosedElements(), p2);
    }
    
    @Override
    public R visitVariable(VariableElement variableElement, P p2) {
        if (variableElement.getKind() != ElementKind.RESOURCE_VARIABLE) {
            return this.scan(variableElement.getEnclosedElements(), p2);
        }
        return this.visitUnknown(variableElement, p2);
    }
    
    @Override
    public R visitExecutable(ExecutableElement executableElement, P p2) {
        return this.scan(executableElement.getParameters(), p2);
    }
    
    @Override
    public R visitTypeParameter(TypeParameterElement typeParameterElement, P p2) {
        return this.scan(typeParameterElement.getEnclosedElements(), p2);
    }
}
